/*
   PMS_MAIN.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1998
   Author: Glenn Bachmann
*/

#include <Pilot.h>
#include <SysEvtMgr.h>
#include <progress.h>
#include <PageMgr.h>
#include "pms.h"
#include "pms_res.h"

typedef enum {
   pageMgrLoaded,
   pageMgrOpened,
   pageMgrUnloaded
} PageMgrStateEnum;

const UInt linesPerPage = 60;

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);
static UInt    MainFormGetPageMgrRefNum (FormPtr formP);
static void    MainFormSetPageMgrRefNum (FormPtr formP, UInt refNum);
static void    MainFormEnableControls (FormPtr formP, PageMgrStateEnum pageMgrState);
static void    MainFormEnableControl (FormPtr formP, Word objectID, Boolean enabled);
static void    MainFormGetPrinter (FormPtr formP, CharPtr printer, UInt len);

// PageMgr function wrappers.
static Err     SampleLoad (FormPtr formP);
static Err     SampleUnload (FormPtr formP);
static Err     SampleVersion (FormPtr formP);
static Err     SampleOpen (FormPtr formP);
static Err     SampleClose (FormPtr formP);
static Err     SamplePrint (FormPtr formP);
static Err     SampleText (FormPtr formP);
static Err     SampleSettings (FormPtr formP);
static Err     SampleNewPage (FormPtr formP);
static Err     SampleVersion (FormPtr formP);

static Char printerStatic [50];

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      
      case ctlSelectEvent:
      {
      // A control button was pressed.
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         MainFormInit (formP);
         FrmDrawForm (formP);
         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr formP)
{
   // Enable the appropriate controls.
   MainFormEnableControls (formP, pageMgrUnloaded);
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainLoadButton:   // User selected Load button.
      {
         SampleLoad (formP);
         handled = true;
         break;
      }

      case MainUnloadButton:   // User selected Unload button.
      {
         SampleUnload (formP);
         handled = true;
         break;
      }

      case MainOpenButton:   // User selected open button.
      {
         SampleOpen (formP);
         handled = true;
         break;
      }

      case MainPrintButton:   // User selected Print button.
      {
         SamplePrint (formP);
         FrmCustomAlert(1100,"Done Printing","","");
         handled = true;
         break;
      }

      case MainCloseButton:   // User selected Close button.
      {
         SampleClose (formP);
         handled = true;
         break;
      }
      
      case MainVersionButton:   // User selected Version button.
      {
         SampleVersion (formP);
         handled = true;
         break;
      }
   }
   return handled;
}

/*
   SampleLoad:
   Sample code for loading the PageMgr shared library.
   Parms:   formP                   Parent form.
   Return:  0                       Success.
            sysErrLibNotFound       PageMgr library not installed.
            sysErrNoFreeRAM         System error loading shared library.
            sysErrNoFreeLibSlots    System error loading shared library.
            pagErrAlreadyLoad       Library has already been loaded.
*/
Err
SampleLoad (FormPtr formP)
{
   // Declare text buffer for messages.
   Char  msg [80];
   UInt  msgSize = sizeof (msg);
   
   // PageMgr may only be loaded once, so we do not try
   // to find it first.  Already loaded is an error in this case.

   UInt refNum = 0;
   Err err = SysLibLoad (pageMgrTypeID, pageMgrCreatorID, &refNum);

   
   // Check result of loading library.
   if (!err)
   {
   // PageMgr loaded successfully.
      // Store ref num.
      MainFormSetPageMgrRefNum (formP, refNum);
      
      // Enable the appropriate controls.
      MainFormEnableControls (formP, pageMgrLoaded);
   }
   else
   {
   // Error loading PageMgr.

      // Notify user of error.
      StrPrintF (msg, "Error %i loading PageMgr.", err);
      FrmCustomAlert (ErrorAlert, msg, NULL, NULL);
   }
   
   // Return result.
   return err;
}

/*
   SampleUnload:
   Unload the PageMgr library.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SampleUnload (FormPtr formP)
{
   // Initialize result.
   Err err = 0;
   
   // Retrieve ref num for PageMgr library.
   UInt refNum = MainFormGetPageMgrRefNum (formP);
   
   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Unload PageMgr library.
   err = SysLibRemove (refNum);

   // Update controls (we are back to unloaded state).
   MainFormEnableControls (formP, pageMgrUnloaded);
   
   // Return result.
   return err;
}


/*
   SampleOpen:
   Sample code for opening the PageMgr shared library.
   Parms:   formP                   Parent form.
   Return:  0                       Success.
            pagErrAlreadyOpen       Library has already been loaded.
*/
Err
SampleOpen (FormPtr formP)
{
   // Declare text buffer for messages.
   Char  msg [80];
   UInt  msgSize = sizeof (msg);
   
   // Retrieve ref num for PageMgr shared library.
   UInt refNum = MainFormGetPageMgrRefNum (formP);
   
   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");



   // Open print manager.
   Err err = PagOpenByConfig (refNum);
   if (!err)
   {
   // PageMgr opened successfully.
      // go get name of printer driver selected   
      PagGetConfigValue(refNum,"DRIVERNAME",printerStatic,sizeof(printerStatic));
      // Update controls.
      MainFormEnableControls (formP, pageMgrOpened);
//      StrPrintF (msg, "PageMgr Opened Successfully!");
//      FrmCustomAlert (SuccessAlert, msg, NULL, NULL);

   }
   else
   {
   // Error opening PageMgr.

      // Notify user of error.
      StrPrintF (msg, "Error %i Opening PageMgr.", err);
      FrmCustomAlert (ErrorAlert, msg, NULL, NULL);
   }
   
   // Return result.
   return err;
}

/*
   SampleClose:
   Close the PageMgr library.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SampleClose (FormPtr formP)
{
   // Initialize result.
   Err err = 0;
   
   // Retrieve ref num for PageMgr library.
   UInt refNum = MainFormGetPageMgrRefNum (formP);
   
   // Check if ref num is valid.
   if (refNum)
   {
   // Ref num is valid.
   
      // Close PageMgr.
      PagClose (refNum);

      // Update controls (we are back to loaded state).
      MainFormEnableControls (formP, pageMgrLoaded);
      
      // Set result to success.
      err = 0;
   }
   else
   {
   // Ref num is not valid.
   
      // Set result to state error.
      err = pagErrState;
   }
   
   // Return result.
   return err;
}

/*
   SamplePrint:
   Print a page using the PageMgr library.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SamplePrint (FormPtr formP)
{
   // Set settings.
   SampleSettings (formP);
   
   // Print Text sample.
   SampleText (formP);
   
   // New page sample.
   SampleNewPage (formP);
   
   // Return success.
   return 0;   
}

/*
   SampleText:
   Exercise text handling.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SampleText (FormPtr formP)
{
// Retrieve PageMgr reference number.

   // Initialize error.
   Err err = 0;
   Char szText[150];
   
   // Retrieve PageMgr reference number.
   UInt refNum = MainFormGetPageMgrRefNum (formP);

   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");


   
   PagFontType font;

   if (!StrCaselessCompare(printerStatic,"PocketJet"))
   {
      const UInt numLines = 50;
      // Print caption.
      StrCopy (font.typeface, "Courier New");
      font.height = 14;
      font.bold = true;
      font.italics = false;
      PagSetFont (refNum, &font);
      err = PagPrintText (refNum, "Print Lines of text");
      if (err)
      {
         StrPrintF(szText,"Error [%d] printing!!!!");
         FrmCustomAlert(ErrorAlert,szText,"","");
         return err;
      }

      StrCopy (font.typeface, "Courier New");
      font.height = 14;
      font.bold = false;
      font.italics = false;
      PagSetFont (refNum, &font);

      // Print subcaption.
      err = PagPrintText (refNum, " (Printer = ");
      if (err)
      {
         StrPrintF(szText,"Error [%d] printing.");
         FrmCustomAlert(ErrorAlert,szText,"","");
         return err;
      }

      err = PagPrintText (refNum, printerStatic);
      if (err)
      {
         StrPrintF(szText,"Error [%d] printing.");
         FrmCustomAlert(ErrorAlert,szText,"","");
         return err;
      }

      err = PagPrintText (refNum, ")");
      if (err)
      {
         StrPrintF(szText,"Error [%d] printing.");
         FrmCustomAlert(ErrorAlert,szText,"","");
         return err;
      }
   
      // Skip a line.
      err = PagNewLine (refNum);
      if (err)
      {
         StrPrintF(szText,"Error [%d] printing.");
         FrmCustomAlert(ErrorAlert,szText,"","");
         return err;
      }
   
      // Restore font to courier 10 normal
      StrCopy (font.typeface, "Courier");
      font.height = 10;
      font.bold = false;
      font.italics = false;
      PagSetFont (refNum, &font);


      // Print 1 page of text.
      for (int i = 0; i < numLines; i++)
      {
         StrPrintF (szText, "%i\tPrinting line of text", i);
         err = PagPrintLine (refNum, szText);
         if (err)
         {
            StrPrintF(szText,"Error [%d] printing line.");
            FrmCustomAlert(ErrorAlert,szText,"","");
            return err;
         }
      }
   }
   else
   {
      const UInt numLines = 40;
      // Set font to Times Roman 14 bold.
      StrCopy (font.typeface, "Times Roman");
      font.height = 10;
      font.bold = true;
      font.italics = false;
      PagSetFont (refNum, &font);
   
      // Print caption.
      err = PagPrintText (refNum, "Print Lines of text");
      if (err)
      {
         StrPrintF(szText,"Error [%d] printing!!!");
         FrmCustomAlert(ErrorAlert,szText,"","");
         return err;
      }

      // Set font to TimesRoman 12 italics.
      font.height = 12;
      font.bold = false;
      font.italics = true;
      PagSetFont (refNum, &font);
   
      // Print subcaption.
      err = PagPrintText (refNum, " (Printer = ");
      if (err)
      {
         StrPrintF(szText,"Error [%d] printing.");
         FrmCustomAlert(ErrorAlert,szText,"","");
         return err;
      }

      err = PagPrintText (refNum, printerStatic);
      if (err)
      {
         StrPrintF(szText,"Error [%d] printing.");
         FrmCustomAlert(ErrorAlert,szText,"","");
         return err;
      }

      err = PagPrintText (refNum, ")");
      if (err)
      {
         StrPrintF(szText,"Error [%d] printing.");
         FrmCustomAlert(ErrorAlert,szText,"","");
         return err;
      }

      // Skip a line.
      err = PagNewLine (refNum);
      if (err)
      {
         StrPrintF(szText,"Error [%d] printing.");
         FrmCustomAlert(ErrorAlert,szText,"","");
         return err;
      }

      // Restore font to courier 10 normal
      StrCopy (font.typeface, "Courier");
      font.height = 10;
      font.bold = false;
      font.italics = false;
      PagSetFont (refNum, &font);
   
      // Print 2 pages of text.
      for (int i = 0; i < numLines; i++)
      {
         StrPrintF (szText, "%i\tPrinting line of textPrinting line of textPrinting line of textPrinting line of textPrinting line of textPrinting line of text", i);
         err = PagPrintLine (refNum, szText);
         if (err)
         {
            StrPrintF(szText,"Error [%d] printing.");
            FrmCustomAlert(ErrorAlert,szText,"","");
            return err;
         }
      }
   }
   
   // Return result.
   return err;
}
   
/*
   SampleNewPage:
   Eject the current page.
   Parms:   refNum      Library reference number.
   Return:  0           Success
*/
Err
SampleNewPage (FormPtr formP)
{
   // Initialize error.
   Err err = 0;
   
   // Retrieve PageMgr reference number.
   UInt refNum = MainFormGetPageMgrRefNum (formP);

   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Set orientation to landscape.
   PagNewPage (refNum);
   
   // Return result.
   return err;
}

/*
   SampleSettings:
   Set default settings for the print job - number of lines per page.
   Parms:   refNum      Library reference number.
   Return:  0           Success
*/
Err
SampleSettings (FormPtr formP)
{
   // Initialize error.
   Err err = 0;
   
   // Retrieve PageMgr reference number.
   UInt refNum = MainFormGetPageMgrRefNum (formP);

   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Set orientation to landscape.
   PagSetOption (refNum, pagOptionMaxLines, linesPerPage);
   
   // Return result.
   return err;
}

/*
   SampleVersion:
   Handle the main form's version command.
   Parms:   formP          Parent form.
   Return:  0              Success
*/
Err
SampleVersion (FormPtr formP)
{
   // Text buffers for messages.
   Char  msg [50];
   UInt  msgSize = sizeof (msg);
   
   // Retrieve reference number.
   UInt refNum = MainFormGetPageMgrRefNum (formP);
   
   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Retrieve version from library.
   DWord dwVer;
   Err err = PagGetLibAPIVersion (refNum, &dwVer);
   
   if (!err)
   {
   // Get API Version returned successfully.
   
      // Inform user of version.
      StrPrintF (msg, "PageMgr Version %lx.", dwVer);
      FrmCustomAlert (SuccessAlert, msg, NULL, NULL);
   }
   else
   {
   // Get API Version returned error.
   
      // Inform user of error.
      StrPrintF (msg, "Error %i retrieving PageMgr version.", err);
      FrmCustomAlert (ErrorAlert, msg, NULL, NULL);
   }
   
   // Return result.
   return err;
}
   
UInt
MainFormGetPageMgrRefNum (FormPtr formP)
{
   // Get gadget index on form.
   Word gadgetIndex = FrmGetObjectIndex (formP, MainDataGadget);
   
   // Retrieve data associated with gadget.
   VoidPtr dataP = FrmGetGadgetData (formP, gadgetIndex);
   
   // Cast data pointer to UInt.
   UInt refNum = (UInt)dataP;
   
   return refNum;
}

void
MainFormSetPageMgrRefNum (FormPtr formP, UInt refNum)
{
   // Get gadget index on form.
   Word gadgetIndex = FrmGetObjectIndex (formP, MainDataGadget);
   
   // Cast ref num to data pointer.
   VoidPtr dataP = (VoidPtr) refNum;
   
   // Set data associated with gadget.
   FrmSetGadgetData (formP, gadgetIndex, dataP);
}

void
MainFormEnableControls (FormPtr formP, PageMgrStateEnum pageMgrState)
{
   // Initialize enable control flags.
   Boolean enableLoad = false;
   Boolean enableOpen = false;
   Boolean enablePrint = false;
   Boolean enableClose = false;
   Boolean enableUnload = false;
   Boolean enableVersion = false;
   
   switch (pageMgrState)
   {
      case pageMgrLoaded :
      {
         enableOpen = true;
         enableVersion = true;
         enableUnload = true;
         break;
      }
      
      case pageMgrOpened :
      {
         enablePrint = true;
         enableClose = true;
         enableVersion = true;
         break;
      }
      
      case pageMgrUnloaded :
      {
         enableLoad = true;
         break;
      }
   }
   
   MainFormEnableControl (formP, MainLoadButton,     enableLoad);
   MainFormEnableControl (formP, MainPrintButton,    enablePrint);
   MainFormEnableControl (formP, MainCloseButton,    enableClose);
   MainFormEnableControl (formP, MainUnloadButton,   enableUnload);
   MainFormEnableControl (formP, MainOpenButton,   enableOpen);
   MainFormEnableControl (formP, MainVersionButton,  enableVersion);
}

void
MainFormEnableControl (FormPtr formP, Word objectID, Boolean enabled)
{
   // Get object index on form.
   Word objectIndex = FrmGetObjectIndex (formP, objectID);
   
   // Get pointer to control.
   ControlPtr controlP = (ControlPtr) FrmGetObjectPtr (formP, objectIndex);
   
   // Set enabled.
   CtlSetEnabled (controlP, enabled);
   
   if (enabled)
   {
      CtlShowControl (controlP);
   }
   else
   {
      CtlHideControl (controlP);
   }
}


unsigned char CallBack(PrgCallbackData* data);
Err PagPrint(char* TextToPrint);

unsigned char CallBack(PrgCallbackData* data)
{
	switch(data->stage)
	{
		// Everything is okay, no errors, just update the msg
		case 0: StrCat(data->textP, data->message); break;
		
		// We got an error!  maybe change the icon to something bad
		case 1: StrCat(data->textP, data->message); break;
		
		default: data->message = "I got a default!";
	}
	return true;
}

// PagPrint()
//
// Takes a block of text, does *all* of the necessary printer
// communication, calls bpmconfig if necessary, prints the text,
// and closes down the printer devices
// ProgMan determines whether or not the Progress Manager comes up
// defaults to true

Err
PagPrint(char* TextToPrint)
{
    Err err = 0;
	Boolean ProgMan = true;
	ProgressPtr Progress;
	// Library reference number
	UInt refNum = 0;
	// Declare text buffer for Error messages.
	Char  msg [80];
	// In case we need to call up BPMConfig
	LocalID id;
	
	if (ProgMan)
	{
		// Initialize and bring up the progress bar
		Progress = PrgStartDialog("Printing...", (unsigned char (*) (PrgCallbackData*)) CallBack);
	}

	// load the print engine
	if (ProgMan)
	{	
		PrgUpdateDialog(Progress, 0, 0, "Loading the Print Engine", true);
	}

	err = SysLibLoad (pageMgrTypeID, pageMgrCreatorID, &refNum);
	// Check result of loading library.
	if (err || refNum == 0)
	{
		// Error loading print manager.
		// Notify user of error.
		if (ProgMan)
		{
			StrPrintF (msg, "Error %i loading Page Manager.", err);
			PrgUpdateDialog(Progress, 1, 1, msg, true);
			PrgStopDialog(Progress, true);	
		}
		return -1;
	}

	
	// Open page manager.
	if (ProgMan)
	{
		PrgUpdateDialog(Progress, 0, 0, "Opening the Printer Device", true);
	}
	err = PagOpenByConfig(refNum);
   
	// If the error is caused by the user not running
	// BPMConfig first, well, force them to run it 
	if ( err == -32758 )
	{
		FrmCustomAlert(ErrorAlert, "You need to configure your printer!  Running BPM Config..", NULL, NULL);
		id = DmFindDatabase(0, "BPMConfig");
		SysUIAppSwitch(0, id, sysAppLaunchCmdNormalLaunch, NULL);
		if (ProgMan)
		{
			PrgUpdateDialog(Progress, 1, 1, msg, true);
			PrgStopDialog(Progress, false);	
		}
		// Close the library
		SysLibRemove (refNum);
		return -2;
	}
	if (err)
	{
		// Error opening page manager.
		// Notify user of error.
		if (ProgMan)
		{
			StrPrintF (msg, "Error %i Opening Page Manager.", err);
			PrgUpdateDialog(Progress, 1, 1, msg, true);
			PrgStopDialog(Progress, false);	
		}
		// Close the Library
		SysLibRemove (refNum);
		return -2;
	}

	// Print the requested text
	if (ProgMan)
	{
		PrgUpdateDialog(Progress, 0, 0, "Sending Print Info.", true);
	}
	
	err = PagPrintParagraph(refNum, TextToPrint, 5);
	if (err)
	{
		// Error opening page manager.
		// Notify user of error.
		if (ProgMan)
		{
			StrPrintF (msg, "Error %i Printing a Line.", err);
			PrgUpdateDialog(Progress, 1, 1, msg, true);
			PrgStopDialog(Progress, false);	
		}
		// Close the printer if possible
		PagClose(refNum);
		// Close the Library
		SysLibRemove (refNum);
		return -3;
	}

	// close the output communications channel
	if (ProgMan)
	{
		PrgUpdateDialog(Progress, 0, 0, "Closing the Printer Device", true);
	}
	err = PagClose (refNum);
	if (err)
	{
		// Error opening page manager.
		// Notify user of error.
		if (ProgMan)
		{
			StrPrintF (msg, "Error %i Closing the Printer Device.", err);
			PrgUpdateDialog(Progress, 1, 1, msg, true);
			PrgStopDialog(Progress, false);	
		}
		// Close the Library
		SysLibRemove (refNum);
		return -4;
	}
	

	// Unload PagMgr library.
	if (ProgMan)
	{
		PrgUpdateDialog(Progress, 0, 0, "Unloading the Print Engine", true);
	}
	err = SysLibRemove (refNum);
	if (err)
	{
		// Error opening page manager.
		// Notify user of error.
		if (ProgMan)
		{
			StrPrintF (msg, "Error %i Closing Page Manager Library.", err);
			PrgUpdateDialog(Progress, 1, 1, msg, true);
			PrgStopDialog(Progress, false);	
		}
		return -5;
	}
	// Kill the Progress Bar
	if (ProgMan)
	{
		PrgStopDialog(Progress, true);
	}

	return err;
}